$VM = Get-SCVirtualMachine -VMMServer "{{ vmm_server }}" -ID "{{ vm_id }}"
$Drive = $VM | Get-SCVirtualDiskDrive | where {$_.ID -eq "{{ drive_id }}"}

$TamanhoGB = {{ size_mb }}/1024
Expand-SCVirtualDiskDrive -VirtualDiskDrive $Drive -VirtualHardDiskSizeGB $TamanhoGB | Out-Null

$Resultado = [PSCustomObject]@{
    Status = 'OK'
}
ConvertTo-Json -Depth 3 $Resultado
